@@ -21,43 +21,80 @@ has case_sensitive => (
     default => 1,
 );
 
+sub _match_as_far_as_possible {
+    my $self = shift;
+    my $path = shift;
+
+    my @got      = $self->tokenize($path->path);
+    my @expected = $self->tokens;
+    my @matched;
+
+    while (@got && @expected) {
+        my $expected = $expected[0];
+        my $got      = $got[0];
+
+        last unless $self->_match_token($got, $expected);
+
+        push @matched, $got;
+        shift @expected;
+        shift @got;
+    }
+
+    return (\@matched, \@got, \@expected);
+}
+
 sub _match {
     my $self = shift;
     my $path = shift;
 
-    my @tokens = $self->tokenize($path->path);
-    my @matched;
+    my ($matched, $got, $expected) = $self->_match_as_far_as_possible($path);
 
-    for my $expected ($self->tokens) {
-        unless (@tokens) {
-            $self->trace(no_tokens => 1, on_token => $expected, path => $path)
-                if $ENV{'PATH_DISPATCHER_TRACE'};
-            return;
-        }
+    return if @$expected; # didn't provide everything necessary
+    return if @$got && !$self->prefix; # had tokens left over
 
-        my $got = shift @tokens;
+    my $leftover = $self->untokenize(@$got);
+    return $matched, $leftover;
+}
 
-        unless ($self->_match_token($got, $expected)) {
-            $self->trace(
-                no_match  => 1,
-                got_token => $got,
-                on_token  => $expected,
-                path      => $path,
-            ) if $ENV{'PATH_DISPATCHER_TRACE'};
-            return;
-        }
+sub complete {
+    my $self = shift;
+    my $path = shift;
 
-        push @matched, $got;
+    my ($matched, $got, $expected) = $self->_match_as_far_as_possible($path);
+    return if @$got > 1; # had tokens leftover
+    return if !@$expected; # consumed all tokens
+
+    my $next = shift @$expected;
+    my $part = @$got ? shift @$got : '';
+    my @completions;
+
+    for my $completion (ref($next) eq 'ARRAY' ? @$next : $next) {
+        next if ref($completion);
+
+        next unless substr($completion, 0, length($part)) eq $part;
+        push @completions, $self->untokenize(@$matched, $completion);
     }
 
-    if (@tokens && !$self->prefix) {
-        $self->trace(tokens_left => \@tokens, path => $path)
-            if $ENV{'PATH_DISPATCHER_TRACE'};
-        return;
-    }
+    return @completions;
+}
 
-    my $leftover = $self->untokenize(@tokens);
-    return \@matched, $leftover;
+sub _each_token {
+    my $self     = shift;
+    my $got      = shift;
+    my $expected = shift;
+    my $callback = shift;
+
+    if (ref($expected) eq 'ARRAY') {
+        for my $alternative (@$expected) {
+            $self->_each_token($got, $alternative, $callback);
+        }
+    }
+    elsif (!ref($expected) || ref($expected) eq 'Regexp') {
+        $callback->($got, $expected);
+    }
+    else {
+        die "Unexpected token '$expected'"; # the irony is not lost on me :)
+    }
 }
 
 sub _match_token {
@@ -65,21 +102,19 @@ sub _match_token {
     my $got      = shift;
     my $expected = shift;
 
-    if (!ref($expected)) {
-        ($got, $expected) = (lc $got, lc $expected) if !$self->case_sensitive;
-        return $got eq $expected;
-    }
-    elsif (ref($expected) eq 'ARRAY') {
-        for my $alternative (@$expected) {
-            return 1 if $self->_match_token($got, $alternative);
+    my $matched = 0;
+    $self->_each_token($got, $expected, sub {
+        my ($g, $e) = @_;
+        if (!ref($e)) {
+            ($g, $e) = (lc $g, lc $e) if !$self->case_sensitive;
+            $matched ||= $g eq $e;
         }
-    }
-    elsif (ref($expected) eq 'Regexp') {
-        return $got =~ $expected;
-    }
-    else {
-        die "Unexpected token '$expected'"; # the irony is not lost on me :)
-    }
+        elsif (ref($e) eq 'Regexp') {
+            $matched ||= $g =~ $e;
+        }
+    });
+
+    return $matched;
 }
 
 sub tokenize {
@@ -91,7 +126,10 @@ sub tokenize {
 sub untokenize {
     my $self   = shift;
     my @tokens = @_;
-    return join $self->delimiter, @tokens;
+    return join $self->delimiter,
+           grep { length }
+           map { split $self->delimiter, $_ }
+           @tokens;
 }
 
 sub readable_attributes {
